#!/bin/sh
#
# /etc/rc.d/rc.S:  System initialization script.
#
# Mostly written by:  Patrick J. Volkerding, <volkerdi@slackware.com>
# Modified for NFS root by Chris Newport 17 June 2002


PATH=/sbin:/usr/sbin:/bin:/usr/bin

# Start devfsd if necessary
if [ -r /dev/.devfsd ]; then
  if [ -x /sbin/devfsd ]; then
    echo "Starting devfs daemon:  /sbin/devfsd /dev"
    /sbin/devfsd /dev
  fi
fi
# This is a horribly messy way of remounting the root filesystem read-write
 
# Normally we rm /etc/mtab here because we cannot mount /proc if it exists
# from a previous boot.
# this will not work because the root filesystem is readonly at this point
# but we need /proc to find out the IP address of the server to remount
# root readonly. Just bail out and tell the user to fix it.
 
if [ -f /etc/mtab ]; then
 echo " FATAL ERROR - please go to my server and delete my etc/mtab "
 halt
fi
 
# the skeleton filesystem has an etc/fstab which only contains proc.
# mount /proc
/bin/mount -avn -t nonfs

# Get our network information from the kernel, we need to be verbose
# in $SRC/linux/net/ipv4/ipconfig.c to get the config details in dmesg.

if [ "`dmesg | grep IP-Config | grep Complete`" = "" ] ;
 then 
 echo "rc.S: bad kernel not reporting net config"
 halt
 else
 MYIP=`ifconfig eth0 | grep inet | cut -f2 -d ':' | cut -f1 -d ' '`
 MYNAME=`hostname`
 MYDOMAIN=`dmesg | grep host | cut -f2 -d ',' | cut -f2 -d '='`
 MYFQDN="$MYNAME.$MYDOMAIN"
 MYDNS=`dmesg | grep nameserver | cut -f4 -d ',' | cut -f2 -d '='`
 BOOTSRV=`dmesg | grep bootserver | cut -f1 -d ',' | cut -f2 -d '='` 
 ROOTSRV=`dmesg | grep rootserver | cut -f2 -d ',' | cut -f2 -d '='` 
 ROOTPATH=`dmesg | grep rootpath | cut -f3 -d ',' | cut -f2 -d '='` 
fi

if [ "$ROOTPATH" = "" ] ; then
 ROOTPATH="/tftpboot/$MYIP"
fi

# remount to get read-write
echo "remounting root filesystem read+write"
/bin/mount -t nfs -w -n -o remount $ROOTSRV:$ROOTPATH /
  if [ $? -gt 0 ] ; then
    echo
    echo "Attempt to remount root device as read-write failed!  This is going to"
    echo "cause serious problems."
    echo "Please fix the server configuration"
    halt
  fi
# If we are running for the first time fixup fstab
if cat /etc/fstab | grep $ROOTSRV:$ROOTPATH
 then break;
 else
 echo "fixing fstab"
  echo "# fstab created automatically for javastation by etc/rc.d/rc.S\n" > /etc/fstab
  echo "none            /proc           proc    defaults   0   0" >> /etc/fstab
  echo "$ROOTSRV:$ROOTPATH  /       nfs     defaults,rsize=8192,wsize=8192  1  1" >> /etc/fstab 
fi
 

# remove /etc/mtab* so mount creates the /etc/mtab file
/bin/rm -f /etc/mtab* /etc/nologin
# fake mtab
ln -s /proc/mounts /etc/mtab
 
# Any remaining filesystems will be mounted later in rc.inet2
 
# echo "end of mount stuff"

# create /etc/hosts if it does not exist
if cat /etc/hosts | grep $MYIP ;
 then
 break 
 else
 echo "# /etc/hosts created automatically by /etc/init.d/rc.S" > /etc/hosts
 echo "" >> /etc/hosts
 echo "127.0.0.1	localhost" >> /etc/hosts
 echo "$MYIP	$MYNAME	$MYFQDN" >> /etc/hosts
fi

echo $MYNAME > /etc/HOSTNAME

# always re-create /etc/resolv.conf
echo "# resolv.conf automatically created by etc/rc.d/rc.S" > /etc/resolv.conf
echo "search $MYDOMAIN" >> /etc/resolv.conf
echo "nameserver $MYDNS" >> /etc/resolv.conf

# enable swapping
# Not yet suported for nfsroot systems
# /sbin/swapon -a

# Start update.
/sbin/update &

# Deleted the fdisk stuff here - cannot fdisk an NFS rootl it should
# be done at the server.
 
# mount file systems in fstab (and create an entry for /)
# but not NFS or SMB because TCP/IP is not yet configured
/sbin/mount -a -v -t nonfs,nosmbfs

# set the correct inittab in place depending on our console type
( if /sbin/checkconsole ; then
     /bin/rm -rf /etc/inittab ; /bin/cp /etc/inittab.system /etc/inittab
     /bin/rm -rf /etc/securetty ; /bin/cp /etc/securetty.system /etc/securetty
  else
     /bin/rm -rf /etc/inittab ; /bin/cp /etc/inittab.serial /etc/inittab
     /bin/rm -rf /etc/securetty ; /bin/cp /etc/securetty.serial /etc/securetty
  fi ) 2> /dev/null
/sbin/telinit q

# Clean up temporary files on the /var volume:
/bin/rm -f /var/run/utmp /var/run/*.pid /var/log/setup/tmp/*

# Looks like we have to create this.
cat /dev/null > /var/run/utmp

# Detect SGI Visual Workstation, since hwclock will make those freeze up:
if fgrep -l Cobalt-APIC /proc/interrupts 1> /dev/null 2> /dev/null ; then
  echo "SGI Visual Workstation detected.  Not running hwclock."
# Set the system time from the hardware clock using hwclock
elif [ -x /sbin/hwclock ]; then
  echo "Setting system time from hardware clock."
  if [ -e /etc/hardwareclock ]; then
    HC=`cat /etc/hardwareclock | while read LINE ; do
      if [ "$LINE" = "UTC" -o "$LINE" = "utc" ]; then
        echo utc
        break;
      elif [ "$LINE" = "localtime" ]; then
        echo localtime
        break;
      fi
    done`
  else
    # Default to localtime if there is no /etc/hardwareclock:
    HC=localtime
  fi
  # /sbin/hwclock --$HC --hctosys
  # The line above *should* work with --localtime, but doesn't.  Omitting
  # the argument defaults it to localtime, so use that.
  if [ "$HC" = "utc" ]; then
    /sbin/hwclock --utc --hctosys
  else
    /sbin/hwclock --hctosys
  fi
fi

#if [ "$ROOTTYPE" = "umsdos" ]; then # we need to update any files added in DOS:
#  echo "Synchronizing UMSDOS directory structure:"
#  echo "  umssync -r99 -v- /"
#  umssync -r99 -v- /
#fi

# Setup the /etc/motd to reflect the current kernel level:
# THIS WIPES ANY CHANGES YOU MAKE TO /ETC/MOTD WITH EACH BOOT.
# COMMENT THIS OUT IF YOU WANT TO MAKE A CUSTOM VERSION.
if [ -f /welcome ]; then
 mv /welcome /etc/motd;
 else
  echo "`/bin/uname -a | /bin/cut -d\  -f1,3`." > /etc/motd
fi

# Configure ISA Plug-and-Play devices:
#if [ -r /etc/isapnp.conf ]; then
#  if [ -x /sbin/isapnp ]; then
#    /sbin/isapnp /etc/isapnp.conf
#  fi
#fi

# NOTE no modules on Javastations.
#
# This loads any kernel modules that are needed.  These might be required to
# use your CD-ROM drive, bus mouse, ethernet card, or other optional hardware.
#if [ -x /etc/rc.d/rc.modules ]; then
#  . /etc/rc.d/rc.modules
#fi

# Initialize PCMCIA devices:
#
# NOTE: This had been closer to the top of this script so that PCMCIA devices
# could be fsck'ed along with the other drives.  This had some unfortunate
# side effects, however, since root isn't yet read-write, and /var might not
# even be mounted the .pid files can't be correctly written in /var/run and
# the pcmcia system can't be correctly shut down.  If you want some PCMCIA
# partition to be mounted at boot (or when the card is inserted) then add
# the appropriate lines to /etc/pcmcia/scsi.opts.
#
#if [ -x /etc/rc.d/rc.pcmcia ] ; then
#  . /etc/rc.d/rc.pcmcia start
#fi

# Run serial port setup script:
# (CAREFUL! This can make some systems hang if the rc.serial script isn't
# set up correctly. If this happens, you may have to edit the file from a
# boot disk)
#
# . /etc/rc.d/rc.serial

# Carry an entropy pool between reboots to improve randomness.
# Load and then save 512 bytes, which is the size of the entropy pool.
if [ -f /etc/random-seed ]; then
  echo "Using /etc/random-seed to initialize /dev/urandom."
  cat /etc/random-seed >/dev/urandom
fi
dd if=/dev/urandom of=/etc/random-seed count=1 bs=512 2> /dev/null

